/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: IdManager_main.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#if defined(WIN32)
#include <windows.h>
#include <io.h>
#include <process.h>
#include <time.h>
#else
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#endif

//#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>

#include "IdManager.h"

#if !defined(NO_LOGGER)
#include "Logger.h"
#endif
#include "ServerCommon.h"
#include "IdManager.h"
#include "PIDFactory.h"
#include "SrvConfAttrHandler.h"

using namespace OpenSOAP;
using namespace std;

//for LOG
static const std::string CLASS_SIG = "IdManager";

int 
main(int argc, char* argv[])
{
#if defined(NO_LOGGER)
  string logFile;
  SrvConfAttrHandler* srvConf = new SrvConfAttrHandler();
  vector<string> values;
  string query = SERVER_CONF_TAG + "/log/path=?";
  if (0 > srvConf->queryXml(query, values)) {
    logFile = OPENSOAP_TMP_PATH + "log/idManager";
  }
  else {
    logFile = values[0] + "idManager.log";
  }
  delete srvConf;

  FILE* outfp = fopen(logFile.c_str(), "w");
  if (!outfp) {
    cerr << "** E **: file open failed.[" << logFile << "]" << endl;
    return 1;
  }
  if (argc <=1) {
    fprintf(outfp, "invalid initial argument. argc=(%d)\n", argc);
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }

  int old;
  int olderr;
  //backup stdout
  old = _dup(1);
  //stdout -> outfp
  if (-1 == _dup2(_fileno(outfp), 1)) {
    fprintf(outfp, "_dup2 to stdout failed.\n");
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }
  //backup stderr
  olderr = _dup(2);
  //stderr -> outfp
  if (-1 == _dup2(_fileno(outfp), 2)) {
    fprintf(outfp, "_dup2 to stderr failed.\n");
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }
#else
  // set log level  
  Logger::InitOpt opt;
  opt.outputLevel = LOG_DEBUG;
  Logger::init(opt);
  //write start message
  Logger::printLog(NULL, " IdManager start", LOG_INFO,
                   0x00100003L, "exe");
#endif


#if !defined(WIN32)
  // make it to daemon & write pid file
#if HAVE_DAEMON
  daemon(0, 1);
#endif

  PIDFactory* pidFactory = new PIDFactory();
  pidFactory->makePID(IDMGR_PID_FILE);
  delete pidFactory;

#endif

  // create instance of IdManager & run
  IdManager idMngr;
  idMngr.run();

#if defined(NO_LOGGER)
  fclose(outfp);
  _dup2(old, 1);
  _dup2(olderr, 2);
#else
  //write stop message
  Logger::printLog(NULL, " IdManager stop", LOG_INFO,
                   0x00100003L, "exe");
  Logger::term();
#endif

  return EXIT_SUCCESS;
}
