/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Logger.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#include <iostream>
#include <locale.h>
#include <syslog.h>
#include <cstdio>
#include <cstdarg>
#include <vector>

#include "StringUtil.h"
//#include "defaultMsg.h"
#include "ServerCommon.h"
#include "SSMLAttrMgrProtocol.h"
//#include "ServerCommon2.h"
#include "SrvConfAttrHandler.h"
#include "Logger.h"

using namespace std;
using namespace OpenSOAP;

//const std::string Logger::QUERY_LOGGING_LEVEL = "/LOG/LOGGING_LEVEL=?";
//const std::string Logger::QUERY_MAX_LEN = "/LOG/MAX_LEN=?";
static const std::string QUERY_MAX_LEN = "/LOG/MAX_LEN=?";

static const std::string QUERY_LOGGING_LEVEL = "/LOG/LEVEL=?";
static const std::string QUERY_CATALOG = "/LOG/CATALOG=?";
static const std::string QUERY_IDENT = "/LOG/IDENT=?";
static const std::string QUERY_OPTION = "/LOG/OPTION=?";
static const std::string QUERY_FACILITY = "/LOG/FACILITY=?";
//const std::string Logger::

static const std::string CLASS_SIG = "Logger::";
static const std::string TAG_ERR = "**[E]**: "; //"[E]:";
static const std::string TAG_WRN = "**[W]**: "; //"[W]:";
static const std::string TAG_INF = "**[I]**: "; //"[I]:";

//initialize
nl_catd Logger::catd = (nl_catd)-1;
int Logger::outputLevel = 4; //=LOG_WARNING //7; //=LOG_DEBUG 
int Logger::outputMaxLen = 256; 
bool Logger::eachTimeLevelCheck = false;
std::string Logger::procSig = "";

Logger::Logger()
{
}
Logger::~Logger()
{
}

//open msgcatalogue
int 
Logger::init()
{
  InitOpt opt;
  return init(opt);
}

#if 1
int 
Logger::init(const InitOpt& opt)
{

  //initial option check.
  if (!opt.procSig.empty()) {
    procSig = NODE_DELMIT + opt.procSig;
  }

  //default values...
  // catalog
  std::string catlog = "opensoap";
  // ident
  const char* openIdent = NULL;
  std::string openIdentStr;
  // option
#if HAVE_LOG_PERROR
  int openOpt = LOG_PERROR|LOG_PID;
#else
  int openOpt = LOG_PID;
#endif
  // facility
  int openFacility = LOG_USER;

  //server conf QƂȂꍇtrue
  bool noRefSrvConf = false;
  // noRefSrvConf
  if (opt.noRefSrvConf) {
    noRefSrvConf = true;
    eachTimeLevelCheck = false;

  }

  SrvConfAttrHandler attrHandler;
  std::vector<std::string> attrs;
  std::string queryStr;

  // server conf check.
  // catlog
  if (!opt.catalogName.empty()) {
    catlog = opt.catalogName;
  }
  else if (!noRefSrvConf) {
    queryStr = procSig + QUERY_CATALOG;
    attrs.clear();
    attrHandler.queryXml(queryStr, attrs);
    if (0 < attrs.size()) {
      catlog = attrs[0];
    }
    else if (!procSig.empty()) {
      queryStr = QUERY_CATALOG;
      attrs.clear();
      attrHandler.queryXml(queryStr, attrs);
      if (0 < attrs.size()) {
	catlog = attrs[0];
      }
    }
  }

  // ident
  if (!opt.openlogIdent.empty()) {
    openIdentStr = opt.openlogIdent;
    openIdent = openIdentStr.c_str();
  }
  else if (!noRefSrvConf) {
    queryStr = procSig + QUERY_IDENT;
    attrs.clear();
    attrHandler.queryXml(queryStr, attrs);
    if (0 < attrs.size()) {
      openIdentStr = attrs[0];
      openIdent = openIdentStr.c_str();
    }
    else if (!procSig.empty()) {
      queryStr = QUERY_IDENT;
      attrs.clear();
      attrHandler.queryXml(queryStr, attrs);
      if (0 < attrs.size()) {
	openIdentStr = attrs[0];
	openIdent = openIdentStr.c_str();
      }
    }
  }

  // option
  if (-1 != opt.openlogOption) {
    openOpt = opt.openlogOption;
  }
  else if (!noRefSrvConf) {
    queryStr = procSig + QUERY_OPTION;
    attrs.clear();
    attrHandler.queryXml(queryStr, attrs);
    if (0 < attrs.size()) {
      int val = 0;
      StringUtil::fromString(attrs[0], val);
      if (val >=0) {
	openOpt = val;
      }
    }
    else if (!procSig.empty()) {
      queryStr = QUERY_OPTION;
      attrs.clear();
      attrHandler.queryXml(queryStr, attrs);
      if (0 < attrs.size()) {
	int val = 0;
	StringUtil::fromString(attrs[0], val);
	if (val >=0) {
	  openOpt = val;
	}
      }
    }
  }

  // facility
  if (-1 != opt.openlogFacility) {
    openFacility = opt.openlogFacility;
  }
  else if (!noRefSrvConf) {
    queryStr = procSig + QUERY_FACILITY;
    attrs.clear();
    attrHandler.queryXml(queryStr, attrs);
    if (0 < attrs.size()) {
      int val = 0;
      StringUtil::fromString(attrs[0], val);
      if (val >=0) {
	openFacility = val;
      }
    }
    else if (!procSig.empty()) {
      queryStr = QUERY_FACILITY;
      attrs.clear();
      attrHandler.queryXml(queryStr, attrs);
      if (0 < attrs.size()) {
	int val = 0;
	StringUtil::fromString(attrs[0], val);
	if (val >=0) {
	  openFacility = val;
	}
      }
    }
  }

  // running level check timing and value.
  //levelCheckAtOnce = opt.levelCheckAtOnce;
  //if (levelCheckAtOnce && (-1 != opt.outputLevel)) {
  if (!noRefSrvConf) {
    //noRefSrvConf=truȅꍇ́Aopt̎wɂ炸false
    eachTimeLevelCheck = opt.eachTimeLevelCheck;
  }
  if (!eachTimeLevelCheck && (-1 != opt.outputLevel)) {
    outputLevel = opt.outputLevel;
  }
  else {
    //Server config check.
    if (!noRefSrvConf) {
      queryAndSetOutputLevel(outputLevel);
    }
  }

  //bZ[WJ^ÕI[v
  //nl_catd catd;
  if (!setlocale(LC_ALL, "")) {
    cerr << CLASS_SIG << "init: setlocale failed." << endl;
  }
  //default catalogue : /usr/share/locale/%L/LC_MESSAGE/opensoap
  if ((catd = catopen(catlog.c_str(), 0)) == (nl_catd)(-1)) {
    cerr << CLASS_SIG << "init: catopen failed." << endl;
    return -1;
  }
  //syslog I[v
  cerr <<"######## ident=[" << openIdent << "]: Opt=(" << openOpt 
       << "): Facility=(" << openFacility << ") ######" << endl;

  openlog(openIdent, openOpt, openFacility);

  //get log output max length par line from server config
  if (!noRefSrvConf) {
    std::vector<std::string> attrs;
    SrvConfAttrHandler attrHandler;
    attrHandler.queryXml(QUERY_MAX_LEN, attrs);
    if (0 < attrs.size()) {
      int val = 0;
      StringUtil::fromString(attrs[0], val);
      if (val > 0) {
	outputMaxLen = val;
      }
    }
  }

  return 0;
}

#else // if 0

int 
Logger::init(const InitOpt& opt)
{

  //initial option check.
  if (!opt.procSig.empty()) {
    procSig = NODE_DELMIT + opt.procSig;
  }

  //server conf QƂȂꍇtrue
  bool noRefSrvConf = false;
  // noRefSrvConf
  if (opt.noRefSrvConf) {
    noRefSrvConf = true;
  }
  // catlog
  std::string catlog = "opensoap"; 
  if (!opt.catalogName.empty()) {
    catlog = opt.catalogName;
  }
  // ident
  const char* openIdent = NULL;
  if (!opt.openlogIdent.empty()) {
    openIdent = opt.openlogIdent.c_str();
  }
  // option
#if HAVE_LOG_PERROR
  int openOpt = LOG_PERROR|LOG_PID;
#else
  int openOpt = LOG_PID;
#endif
  if (-1 != opt.openlogOption) {
    openOpt = opt.openlogOption;
  }
  // facility
  int openFacility = LOG_USER;
  if (-1 != opt.openlogFacility) {
    openFacility = opt.openlogFacility;
  }
  // running level check timing and value.
  //levelCheckAtOnce = opt.levelCheckAtOnce;
  //if (levelCheckAtOnce && (-1 != opt.outputLevel)) {
  eachTimeLevelCheck = opt.eachTimeLevelCheck;
  if (eachTimeLevelCheck && (-1 != opt.outputLevel)) {
    outputLevel = opt.outputLevel;
  }
  else {
    //Server config check.
    if (!noRefSrvConf) {
      queryAndSetOutputLevel(outputLevel);
    }
  }

  //bZ[WJ^ÕI[v
  //nl_catd catd;
  if (!setlocale(LC_ALL, "")) {
    cerr << CLASS_SIG << "init: setlocale failed." << endl;
  }
  //default catalogue : /usr/share/locale/%L/LC_MESSAGE/opensoap
  if ((catd = catopen(catlog.c_str(), 0)) == (nl_catd)(-1)) {
    cerr << CLASS_SIG << "init: catopen failed." << endl;
    return -1;
  }
  //syslog I[v
  openlog(openIdent, openOpt, openFacility);

  //get log output max length par line from server config
  if (!noRefSrvConf) {
    std::vector<std::string> attrs;
    SrvConfAttrHandler attrHandler;
    attrHandler.queryXml(QUERY_MAX_LEN, attrs);
    if (0 < attrs.size()) {
      int val = 0;
      StringUtil::fromString(attrs[0], val);
      //int val = atoi(attrs[0].c_str());
      if (val > 0) {
	outputMaxLen = val;
      }
    }
  }
  else {
    //server conf QƂȂꍇeachTimeLevelCheckfalseɂĂB
    ////server conf QƂȂꍇlevelCheckAtOncetrueɂĂB
    eachTimeLevelCheck = false;
    //levelCheckAtOnce = true;
  }

  return 0;
}
#endif // if 0

//close msgcatalogue
int 
Logger::term()
{
  //syslogN[Y
  closelog();
  //bZ[WJ^ÕN[Y
  catclose(catd);

  return 0;
}

//for development phase.
void 
Logger::printLog(const char* const optionMsg,
		 const std::string& defaultMsg, 
		 int level, int errCode, ...)
{
  //int runningLevel = OpenSOAP::LOG_WARNING;
  int runningLevel = outputLevel;

  //sɒo̓x`FbNꍇB
  //if (!levelCheckAtOnce) {
  //  queryAndSetOutputLevel(runningLevel);
  //}
  if (eachTimeLevelCheck) {
    queryAndSetOutputLevel(runningLevel);
  }
  
  //sw背x傫l(=dvx)̏ꍇ͏IB
  if (runningLevel < level) {
    return;
  }
  //get msgcatalog 
  const char* catmsg = getErrorMsg(errCode, defaultMsg);
  if (NULL == catmsg) {
    cerr << CLASS_SIG << "printLog: no msg catalogue. errCode=["
	 << errCode << "]" << endl;
    return;
  }
  va_list args;
  va_start(args, errCode);
  char* buf = new char[outputMaxLen];
  vsnprintf(buf, outputMaxLen, catmsg, args);
  va_end(args);

  std::string levelTag;
  std::string optTag;
  switch (level) {
  case LOG_ERR:
    levelTag = TAG_ERR;
    break;
  case LOG_WARNING:
    levelTag = TAG_WRN;
    break;
  case LOG_INFO:
    levelTag = TAG_INF;
    break;
  default:
    break;
  }
  if (optionMsg) {
    optTag = optionMsg;
  }
  syslog(level, "%s%s%s",
	 levelTag.c_str(),
	 optTag.c_str(),
	 buf);

  delete[] buf;
}


//for development phase.
void 
Logger::printLogRefInitLevel(const char* const optionMsg,
			     const std::string& defaultMsg, 
			     int level, int errCode, ...)
{
  //int runningLevel = OpenSOAP::LOG_WARNING;
  int runningLevel = outputLevel;

  //sw背x傫l(=dvx)̏ꍇ͏IB
  if (runningLevel < level) {
    return;
  }
  //get msgcatalog 
  const char* catmsg = getErrorMsg(errCode, defaultMsg);
  if (NULL == catmsg) {
    cerr << CLASS_SIG << "printLog: no msg catalogue. errCode=["
	 << errCode << "]" << endl;
    return;
  }
  va_list args;
  va_start(args, errCode);
  char* buf = new char[outputMaxLen];
  vsnprintf(buf, outputMaxLen, catmsg, args);
  va_end(args);

  std::string levelTag;
  std::string optTag;
  switch (level) {
  case LOG_ERR:
    levelTag = TAG_ERR;
    break;
  case LOG_WARNING:
    levelTag = TAG_WRN;
    break;
  case LOG_INFO:
    levelTag = TAG_INF;
    break;
  default:
    break;
  }
  if (optionMsg) {
    optTag = optionMsg;
  }
  syslog(level, "%s%s%s",
	 levelTag.c_str(),
	 optTag.c_str(),
	 buf);

  delete[] buf;
}

char*
Logger::getErrorMsg(int errCode, const std::string& defaultMsg)
{
  char *ret = NULL;
  div_t ids = ::div(errCode, 0x10000L);
  int set_id = ids.quot;
  int msg_id = ids.rem;

  ret = catgets(catd, set_id, msg_id, defaultMsg.c_str());
  
  return ret;
}

void 
Logger::queryAndSetOutputLevel(int& level)
{
  SrvConfAttrHandler attrHandler;
  std::vector<std::string> attrs;
  //get log output level from server config
  //attrHandler.queryXml(QUERY_LOGGING_LEVEL, attrs);
  std::string queryStr = procSig + QUERY_LOGGING_LEVEL;
  attrHandler.queryXml(queryStr, attrs);
  if (0 < attrs.size()) {
    //int val = atoi(attrs[0].c_str());
    int val = 0;
    StringUtil::fromString(attrs[0], val);
    if (val >= 0) {
      level = val;
    }
  }
  else if (!procSig.empty()) {
    queryStr = QUERY_LOGGING_LEVEL;
    attrs.clear();
    attrHandler.queryXml(queryStr, attrs);
    if (0 < attrs.size()) {
      int val = 0;
      StringUtil::fromString(attrs[0], val);
      if (val >= 0) {
	level = val;
      }
    }
  }
}

// End of Logger.cpp
