/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: SignatureFunc.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#include <string>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /* HAVE_CONFIG_H */

/* libxml2 */
#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */

#include "fio.h"
#include "ServerCommon.h"
#include "SrvConfAttrHandler.h"
#include "OpenSOAP/OpenSOAP.h"
#include "OpenSOAP/Envelope.h"
#include "OpenSOAP/Security.h"

using namespace OpenSOAP;
using namespace std;

#if 0
bool
getPrivKeyName(string& privKeyName)
{
  SrvConfAttrHandler srvConfAttrHndl;
  vector<std::string> values;
  string query = SrvConfAttrHandler::SERVER_CONF_TAG + "/keys/path=?";
  if (0 > srvConfAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***: keys/path get failed." << endl;
    return false;
  }
  privKeyName = values[0];
  if (privKeyName[privKeyName.length()] != '/') {
    privKeyName += "/";
  }
  privKeyName += "privKey.pem";
  return true;
}
#endif

#if 0
bool
getAddSignature(std::string& addSigValue)
{
  SrvConfAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  std::string query;
  std::string addSignatureValue;
  
  query = SrvConfAttrHandler::SERVER_CONF_TAG + "/add_signature=?";
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***" << endl;
    return false;
  }
  addSigValue = values[0];
  return true;
}
#endif

#if 0
// check "conf/server/signature.conf" to sign or not
bool
needSignature()
{
  string addSignatureValue;
  if (!getAddSignature(addSignatureValue)) {
    cerr << "*** ERROR ***: getAddSignature() failed." << endl;
    return false;
  }

  cerr << "SignatureFunc::needSignature:"
       << "addSignatureValue=[" << addSignatureValue << "]" << endl;
  
  if(addSignatureValue == "true") {
    return true;
  }
  else {
    return false;
  }
}
#endif

#if 0
bool
addSignatureToFile(std::string fileID)
{
  // add signature
  string filename;
  if (!getMessageSpoolPath(filename)) {
    cerr << "*** ERROR ***: getMessageSpoolPath() failed." << endl;
    return false;
  }
  filename += fileID;

  string szPrivKName;
  if (!getPrivKeyName(szPrivKName)) {
    cerr << "*** ERROR ***: getPrivKeyName() failed." << endl;
    return false;
  }

  int nRet;
  OpenSOAPEnvelopePtr env = NULL;
  
  /* OpenSOAP API  */
  nRet = OpenSOAPInitialize(NULL);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
  /* Envelopeե */
  nRet = LoadEnvelope(filename.c_str(), &env);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
  /* ̾ղ */
  nRet = OpenSOAPSecAddSignWithFile(env, OPENSOAP_HA_SHA,
				    szPrivKName.c_str(), NULL);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
  
  /* ̾Τ¸ */
  nRet = SaveEnvelope(env, filename.c_str());
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
FuncEnd:
  OpenSOAPUltimate();
  
  return true;
}
#endif

bool
addSignatureToString(std::string& soapMsg, const std::string& keyName)
{
  int nRet;
  OpenSOAPEnvelopePtr env = NULL;
  
  /* OpenSOAP API  */
  nRet = OpenSOAPInitialize(NULL);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
  
  /* Envelopeʸ */
  nRet = LoadEnvelope(soapMsg, &env);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    cerr << "addSignatureToString(): "
	 << "LoadEnvelpe fail." << endl;
    return false;
  }
  
  /* ̾ղ */
  nRet = OpenSOAPSecAddSignWithFile(env, OPENSOAP_HA_SHA,
				    keyName.c_str(), NULL);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    cerr << "addSignatureToString(): "
	 << "OpenSOAPSecAddSignWithFile fail." << endl;
    return false;
  }
  
  /* ̾Τ¸ */
  nRet = SaveEnvelope(env, soapMsg);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    cerr << "addSignatureToString(): "
	 << "SaveEnvelope fail." << endl;
    return false;
  }
  
FuncEnd:
  OpenSOAPUltimate();
  
  return true;
}
